#ifndef IFXAUDIT_H
#define IFXAUDIT_H

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
    SYSLOG,
    STDOUT,
    SYSLOG_AND_STDOUT,
    DISKFILE
} OutputDestination;

typedef enum {
    FAILURE_LOG = 0,
    APPLICATION,
    MANAGEMENT
} AuditType;

typedef enum {
    AUDIT_SUCCESS = 0,
    IFXAUDIT_NOT_INITIALIZED,
    INVALID_MANDATORY_PROPERTIES,
    INVALID_OPTIONAL_PROPERTIES,
    INVALID_PART_C,
    AUDIT_FAILURE
} IFxAuditReturnValue;

// Note:
// These enums are currently written this way to make code clearer and conform to readability standards.
// On the Windows side, these enums are written in CamelCase.
// It is likely that we will need to write the data in that format, to simplify centralized log searching.
// Developers searching their own logs may be confused why they can find "UserManagement" but not "USER_MANAGMENT".
// We should make it clear to users that this is being done,
// and wrappers written around this library (e.g. in Go or Python), should rename the enums to limit confusion.
typedef enum {
    RESULT_TYPE_NOT_SET = 0,
    SUCCESS,
    CLIENT_ERROR,
    FAILURE,
    TIMEOUT
} IFxAuditResultType;

typedef enum
{
    OTHER,
    USER_MANAGEMENT,
    GROUP_MANAGEMENT,
    AUTHENTICATION,
    AUTHORIZATION,
    ROLE_MANAGEMENT,
    APPLICATION_MANAGEMENT,
    KEY_MANAGEMENT,
    DIRECTORY_MANAGEMENT,
    RESORUCE_MANAGEMENT,
    POLICY_MANAGEMENT,
    DEVICE_MANAGEMENT,
    ENTITLEMENT_MANAGEMENT,
    PASSWORD_MANAGEMENT,
    IDENTITY_PROTECTION,
    OBJECT_MANAGEMENT,
    PROVISIONING_MANAGEMENT
} IFxAuditEventCategories;

typedef enum {
    UPN,
    PUID,
    OBJECT_ID,
    CERTIFICATE,
    CLAIM,
    USERNAME,
    KEY_NAME,
    SUBSCRIPTION_ID,
    APPLICATION_ID
} IFxAuditCallerIdType;

// Details of the entity invoking the operation being audited.
typedef struct {
    IFxAuditCallerIdType type;
    char* value;
} ifxaudit_caller_id_t;

// Details of the resource impacted by the operation being audited.
typedef struct {
    char* type;
    char* name;
} ifxaudit_target_resource_t;

// Required audit properties.
// All of these must have a meaningful value. No list may be empty.
// "Part B" is composed of the required properties and the optional properties.
typedef struct {
    char* operation_name;
    IFxAuditResultType result_type;

    // The caller is expected to ensure that these are categories are unique
    IFxAuditEventCategories* audit_categories_set;
    int audit_categories_set_size;

    ifxaudit_caller_id_t* caller_id_list;
    int caller_id_list_size;

    ifxaudit_target_resource_t* target_resource_list;
    int target_resource_list_size;
} ifxaudit_rqd_props_t;

// Optional audit properties. Any of these, or the whole struct, may be NULL
typedef struct {
    char* result_desc;
    char* caller_ip_addr;
    char* ncloud;
    char* request_id;
    char* caller_display_name;
} ifxaudit_opt_props_t;

// A team may elect to include a "Part C", custom fields relevant to their logs.
// IFx only supports simple types for Part C - bool, int, float, strings, etc.
// Other IFx libraries use Bond to implement this.
// As there is no Bond implementation in C, we use a list of name/value pairs.
typedef struct {
    char** part_c_names_list;
    char** part_c_values_list;
    int part_c_list_size;
} ifxaudit_part_c_t;

// TODO move this to a file, read from there instead
typedef struct {
    int heartbeat_interval_seconds;
    OutputDestination dest;
    char* file;
} ifxaudit_audit_spec_t;

// Caller is responsible for cleaning up all memory.
// All are thread-safe.
IFxAuditReturnValue ifxaudit_log_audit(
    AuditType audit_type,
    const ifxaudit_rqd_props_t* rqd_props,
    const ifxaudit_opt_props_t* opt_props,
    const ifxaudit_part_c_t* part_c);

// Format audit properties for output to syslog
// Caller should pass in an empty char**,
// which on success will point to the data to be outputted.
// On success, the caller should free *formatted_data_ptr.
// On failure, *formatted_data_ptr will not need to be freed
IFxAuditReturnValue ifxaudit_format_audit(
    AuditType audit_type,
    const ifxaudit_rqd_props_t* rqd_props,
    const ifxaudit_opt_props_t* opt_props,
    const ifxaudit_part_c_t* part_c,
    char** formatted_data_ptr); // TODO: change this to bytes later?

// Spins up thread to send IFxAudit heartbeat on specified interval.
IFxAuditReturnValue ifxaudit_initialize(const ifxaudit_audit_spec_t* audit_spec);

void ifxaudit_dispose();

#ifdef __cplusplus
} // Extern "C" block
#endif

#endif